<?php

namespace Drupal\commerce_bpay_v2\Controller;

use Drupal\commerce_bpay_v2\Interfaces\BPayRedirectControllerInterface;
use Drupal\commerce_order\Entity\Order;
use Drupal\commerce_bpay_v2\Common\Helper;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\commerce_payment\Controller\PaymentCheckoutController as CommercePaymentCheckoutController;
use Zend\Diactoros\Response\JsonResponse;

/**
 * Class BPayRedirectController.
 */
class BPayRedirectController extends CommercePaymentCheckoutController implements BPayRedirectControllerInterface {

    /**
     * Notify page callback.
     *
     * @param \Symfony\Component\HttpFoundation\Request $request
     *   Request.
     *
     * @return JsonResponse
     *   The response.
     *
     * @throws \Drupal\Core\Entity\EntityMalformedException
     * @throws \Drupal\Core\TypedData\Exception\MissingDataException
     */
    public function notify(Request $request, RouteMatchInterface $route_match) {
        if (! $request->isMethod('post')) {
            exit('Ohhh nooo, are you a hacker?');
        }

        $data = $request->request->get('data');
        $key = $request->request->get('key');

        if (!$data) {
            return  new JsonResponse(['code' => 30, 'text' => 'Data Empty']);
        }

        if (!$key) {
            return  new JsonResponse(['code' => 30, 'text' => 'Key Empty']);
        }
        $jsondata = json_decode(base64_decode($data));
        $order = $this->getOrder($jsondata->order_id);
        $payment_gateway = $order->get('payment_gateway')->entity;
        $vrfsign = hash('sha256',hash('sha256', $data).hash('sha256', $payment_gateway->get('configuration')['signature']));
        if ($key !== $vrfsign) {
            return  new JsonResponse(['code' => 30, 'text' => 'Signature Error']);
        }

        $payment_gateway_plugin = $payment_gateway->getPlugin();
        if (!$payment_gateway_plugin instanceof OffsitePaymentGatewayInterface) {
            throw new AccessException('The payment gateway for the order does not implement ' . OffsitePaymentGatewayInterface::class);
        }

        if ((string)$jsondata->comand === "pay") {
            $payment_gateway_plugin->onReturn($order, $request);
            return  new JsonResponse([
                'code' => 100,
                'text' => 'success'
            ]);
        }
    }


    /**
     * @param int $order_id
     * @return \Drupal\Core\Entity\Entity|\Drupal\Core\Entity\EntityInterface|null
     * Получаем оредр
     */
    public function getOrder($order_id)
    {
        return Order::load($order_id);
    }

    /**
     * @param Request $request
     * @return string[]
     * Страница с успешной оплатой
     */
    public function success(Request $request)
    {
        $order_id = $request->get('commerce_order');
        if (!$order_id) {
            $element = [
                '#markup' => 'Произошла ошибка, ордер пустой'
            ];
        }else{
            $element = [
                '#markup' => ''.t("Спасибо за заказ. Оператор свяжется с вами, в течении 10 мин").' <br /> '.t("Ваш номер заказа: <b>$order_id</b>").''
            ];
        }

        return $element;
    }

    /**
     * @param Request $request
     * @return string[]
     * Страница с ошибкой
     */
    public function fail(Request $request)
    {
        $order_id = $request->get('commerce_order');
        $element = [
            '#markup' => 'Произошла ошибка при оплате, или вы ушли с страницы <br />'.t("Ваш номер заказа: <b>$order_id</b>").''
        ];
        return $element;
    }
}
